/*
 * colours.cpp: Colour wrangling.
 * Part of Storming Ions (a screensaver).
 * Copyright (C) 2007 W. Ross Younger.
 * See the included file "COPYING" for licensing information.
 */

#include "colours.h"
#include <math.h>

int gColourCount;
COLORREF *colours;
HPEN *pens;
HBRUSH *brushes;
COLORREF BLACK, WHITE;

/* HSB (HSL) to RGB conversion - taken from xlockmore */
static void hsb2rgb(double H, double S, double B,
					unsigned char *r, unsigned char *g, unsigned char *b)
{
	int         i;
	double      f, bb;
	unsigned char p, q, t;

	H -= floor(H);
	H *= 6.0;
	i = (int) floor(H);	/* 0..5 */
	f = H - (float) i;	/* f = fractional part of H */
	bb = 255.0 * B;
	p = (unsigned char) (bb * (1.0 - S));
	q = (unsigned char) (bb * (1.0 - (S * f)));
	t = (unsigned char) (bb * (1.0 - (S * (1.0 - f))));
	switch (i) {
		case 0:
			*r = (unsigned char) bb;
			*g = t;
			*b = p;
			break;
		case 1:
			*r = q;
			*g = (unsigned char) bb;
			*b = p;
			break;
		case 2:
			*r = p;
			*g = (unsigned char) bb;
			*b = t;
			break;
		case 3:
			*r = p;
			*g = q;
			*b = (unsigned char) bb;
			break;
		case 4:
			*r = t;
			*g = p;
			*b = (unsigned char) bb;
			break;
		case 5:
			*r = (unsigned char) bb;
			*g = p;
			*b = q;
			break;
	}
}



void select_colour(HDC hdc,int idx)
{
	if ( (idx<0) | (idx>=gColourCount))
		idx=0;
	SelectObject(hdc,pens[idx]);
	SelectObject(hdc,brushes[idx]);
}


int init_colours(int count)
{
	gColourCount = count;
	colours = new COLORREF[count];
	pens = new HPEN[count];
	brushes = new HBRUSH[count];

	double H=0.0,S=1.0,L=1.0;
	unsigned char r=0,g=0,b=0;
	int i;
	COLORREF cr;
	double step = (1.0/count);
	HPEN hpen;
	HBRUSH hbrush;
	
	for (i=0; i < count; i++) {
		H += step;
		hsb2rgb(H,S,L,&r,&g,&b);
		cr=RGB(r,g,b);
		colours[i]=cr;

		hpen=CreatePen(PS_SOLID,0,cr);
		if (hpen == NULL) return -1;
		pens[i]=hpen;

		hbrush=CreateSolidBrush(cr);
		if (hbrush==NULL) return -1;
		brushes[i]=hbrush;
	}
	BLACK = RGB(0,0,0);
	WHITE = RGB(255,255,255);
	return 0;
}


void cleanup_colours(void)
{
	int i;
	for (i=0; i < gColourCount; i++) {
		DeleteObject(pens[i]);
		DeleteObject(brushes[i]);
	}
	delete[] colours;
	delete[] pens;
	delete[] brushes;
	gColourCount=0;
}

