/*
 * config.h: Configuration management interface
 * Part of Storming Ions (a screensaver).
 * Copyright (C) 2007 W. Ross Younger.
 * See the included file "COPYING" for licensing information.
 */

#ifndef _CONFIG_H
#define _CONFIG_H

/* T_FLOAT defines what I call a _constrained_ float - a fixed-denominator rational */
typedef enum config_type { T_BOOL, T_INT, T_FLOAT, T_SELECT } config_type;

struct keyval {
	const char *key;
	const int val;
	keyval() : key(""), val(0) {}
	keyval(const char *k, int v) : key(k), val(v) {}
};

typedef struct keyval keyval;

#define KV(k,v) new struct keyval(k,v)

struct config_item {
	const char	*name; /* Should not be malloced. Unique - used as registry key. */
	const char	*help; /* ditto */
	config_type type;

	int			val; /* for all types */
	int			ctlid; /* dynamically allocated windows control ID, when the dialog is running */
	const int	_default;

	const int _min, _max, _step; /* for int, float - min/max/step for config display */
	const int _denom; /* for float - denominator */
	const keyval **_kv; /* keyval pairs - NULL unless a SELECT type;
					     when present, MUST be terminated by {0,0} */

	config_item(const char *name_, const char *help_, config_type type_,
				const int default_, const int min_, const int max_, const int step_,
				const int denom_,
				const keyval **kv_) : name(name_), help(help_), type(type_), 
				_default(default_), _min(min_), _max(max_), _step(step_),
				_denom(denom_), _kv(kv_) {/* Yeuch */}
};

#define  CONFIG_BOOL(name,help,def) new config_item(name,help,T_BOOL,def,0,0,0,0,0)
#define   CONFIG_INT(n,h,def, min,max,step) new config_item(n,h,T_INT,def,min,max,step,0,0)
#define CONFIG_FLOAT(n,h,def, min,max,step,denom) new config_item(n,h,T_FLOAT,def,min,max,step,denom,0)
#define   CONFIG_SEL(n,h,def, kv) new config_item(n,h,T_SELECT,def, 0,0,0,0, kv)

typedef struct _config_context *config_context;

extern unsigned n_configdefs;
config_item **configs();
int config_save();
int config_find_index(const char *name);
int config_get(const char *name);
int config_get(int index);
double config_get_float(const char *name);

#endif